@@ -3,13 +3,9 @@ package Audio::Wav::Write;
 use strict;
 use FileHandle;
 use Audio::Wav::Write::Header;
+
 use vars qw( $VERSION );
-$VERSION = '0.03';
-
-
-my @needed = qw( bits_sample channels sample_rate );
-my @wanted = qw( block_align bytes_sec info);
-
+$VERSION = '0.06';
 
 =head1 NAME
 
@@ -25,10 +21,13 @@ Audio::Wav::Write - Module for writing Microsoft WAV files.
     my $bits_sample = 16;
 
     my $details = {
-		    'bits_sample'	=> $bits_sample,
-		    'sample_rate'	=> $sample_rate,
-		    'channels'		=> 1,
-		  };
+	'bits_sample'	=> $bits_sample,
+	'sample_rate'	=> $sample_rate,
+	'channels'	=> 1,
+	# if you'd like this module not to use a write cache, uncomment the next line
+	#'no_cache'	=> 1,
+
+    };
 
     my $write = $wav -> write( 'testout.wav', $details );
 
@@ -77,15 +76,23 @@ sub new {
 
     my $handle = new FileHandle ">$out_file";
 
+    my $use_cache = 1;
+    if ( ref( $details ) eq 'HASH' && exists( $details -> {'no_cache'} ) ) {
+	my $no_cache = delete $details -> {'no_cache'};
+	$use_cache = 0 if $no_cache;
+    }
+
     my $self =	{
-		'write_cache'	=> undef,
-		'out_file'	=> $out_file,
-		'cache_size'	=> 4096,
-		'handle'	=> $handle,
-		'details'	=> $details,
-		'block_align'	=> $details -> {'block_align'},
-		'tools'		=> $tools,
-	        };
+	'use_cache'	=> $use_cache,
+	'write_cache'	=> undef,
+	'out_file'	=> $out_file,
+	'cache_size'	=> 4096,
+	'handle'	=> $handle,
+	'details'	=> $details,
+	'block_align'	=> $details -> {'block_align'},
+	'tools'		=> $tools,
+	'done_finish'	=> 0,
+    };
 
     bless $self, $class;
 
@@ -111,6 +118,13 @@ sub new {
     return $self;
 }
 
+sub DESTROY {
+    my $self = shift;
+    return unless $self;
+    return if $self -> {'done_finish'};
+    $self -> finish();
+}
+
 =head2 finish
 
 Finishes off & closes the current wav file.
@@ -121,17 +135,15 @@ Finishes off & closes the current wav file.
 
 sub finish {
     my $self = shift;
-    $self -> _purge_cache();
-    my $length = $self -> {'pos'};
-    my $header = $self -> {'header'};
-    $header -> finish( $length );
+    $self -> _purge_cache() if $self -> {'use_cache'};
+    $self -> {'header'} -> finish( $self -> {'pos'} );
     $self -> {'handle'} -> close();
-    my $filename = $self -> {'out_file'};
+    $self -> {'done_finish'} = 1;
 }
 
 =head2 add_cue
 
-Adds a cue point to the wav file.
+Adds a cue point to the wav file. If $sample is undefined then the position will be the current position (end of all data written so far).
 
     # $byte_offset for 01 compatibility mode
     $write -> add_cue( $sample, "label", "note"  );
@@ -143,10 +155,15 @@ sub add_cue {
     my $pos = shift;
     my $label = shift;
     my $note = shift;
-    $pos /= $self -> {'details'} -> {'block_align'} if $self -> {'tools'} -> is_01compatible();
+    my $block_align = $self -> {'details'} -> {'block_align'};
+    if ( defined $pos ) {
+	$pos /= $block_align if $self -> {'tools'} -> is_01compatible();
+    } else {
+	$pos = $self -> {'pos'} / $block_align;
+    }
     my $output = {
-		    'pos'	=> $pos,
-		 };
+	'pos'	=> $pos,
+    };
     $output -> {'label'} = $label if $label;
     $output -> {'note'} = $note if $note;
     $self -> {'header'} -> add_cue( $output );
@@ -157,15 +174,15 @@ sub add_cue {
 All parameters are optional.
 
     my %info = (
-		'midi_pitch_fraction'	=> 0,
-		'smpte_format'		=> 0,
-		'smpte_offset'		=> 0,
-		'product'		=> 0,
-		'sample_period'		=> 0,
-		'manufacturer'		=> 0,
-		'sample_data'		=> 0,
-		'midi_unity_note'	=> 65,
-	       );
+	'midi_pitch_fraction'	=> 0,
+	'smpte_format'		=> 0,
+	'smpte_offset'		=> 0,
+	'product'		=> 0,
+	'sample_period'		=> 0,
+	'manufacturer'		=> 0,
+	'sample_data'		=> 0,
+	'midi_unity_note'	=> 65,
+    );
     $write -> set_sampler_info( %info );
 
 =cut
@@ -182,11 +199,11 @@ All parameters are optional except start & end.
     my $length = $read -> length_samples();
     my( $third, $twothirds ) = map int( $length / $_ ), ( 3, 1.5 );
     my %loop = (
-		'start'			=> $third,
-		'end'			=> $twothirds,
-		'fraction'		=> 0,
-		'type'			=> 0,
-	       );
+	'start'			=> $third,
+	'end'			=> $twothirds,
+	'fraction'		=> 0,
+	'type'			=> 0,
+    );
     $write -> add_sampler_loop( %loop );
 
 =cut
@@ -257,7 +274,7 @@ sub write {
 
 =head2 write_raw
 
-Adds a some pre-packed data to the current file.
+Adds some pre-packed data to the current file.
 
     $write -> write_raw( $data, $data_length );
 
@@ -272,26 +289,46 @@ sub write_raw {
     my $self = shift;
     my $data = shift;
     my $len = shift;
-    my $no_cache = shift;
     $len = length( $data ) unless $len;
     return unless $len;
     my $wrote = $len;
-    if ( $no_cache ) {
-	$wrote = syswrite $self -> {'handle'}, $data, $len;
-    } else {
+    if ( $self -> {'use_cache'} ) {
 	$self -> {'write_cache'} .= $data;
 	my $cache_len = length( $self -> {'write_cache'} );
 	$self -> _purge_cache( $cache_len ) unless $cache_len < $self -> {'cache_size'};
+    } else {
+	$wrote = syswrite $self -> {'handle'}, $data, $len;
     }
 
     $self -> {'pos'} += $wrote;
     return $wrote;
 }
 
+=head2 write_raw_samples
+
+Adds some pre-packed data to the current file, returns number of samples written.
+
+    $write -> write_raw_samples( $data, $data_length );
+
+Where;
+
+    $data is the packed data
+    $data_length (optional) is the length in bytes of the data
+
+=cut
+
+sub write_raw_samples {
+    my $self = shift;
+    my $written = $self -> write_raw( @_ );
+    return $written / $self -> {'details'} -> {'block_align'};
+}
+
+####################
+
 sub _start_file {
     my $self = shift;
     my( $file, $details, $tools, $handle ) = map $self -> {$_}, qw( out_file details tools handle );
-    my $header = Audio::Wav::Write::Header -> new( $file, $details, $tools, $handle, $self );
+    my $header = Audio::Wav::Write::Header -> new( $file, $details, $tools, $handle );
     $self -> {'header'} = $header;
     my $data = $header -> start();
     $self -> write_raw( $data );
@@ -308,13 +345,14 @@ sub _purge_cache {
     $self -> {'write_cache'} = undef;
 }
 
-####################
-
 sub _init {
     my $self = shift;
     my $details = $self -> {'details'};
     my $output = {};
     my @missing;
+    my @needed = ( 'bits_sample', 'channels', 'sample_rate' );
+    my @wanted = ( 'block_align', 'bytes_sec', 'info', 'wave-ex' );
+
     foreach my $need ( @needed ) {
 	if ( exists( $details -> {$need} ) && $details -> {$need} ) {
 	    $output -> {$need} = $details -> {$need};
@@ -383,8 +421,8 @@ sub _error {
 
 =head1 AUTHORS
 
-    Nick Peskett <cpan@peskett.com>.
-    Kurt George Gjerde <kurt.gjerde@media.uib.no>. (from 0.02)
+    Nick Peskett (see http://www.peskett.co.uk/ for contact details).
+    Kurt George Gjerde <kurt.gjerde@media.uib.no>. (0.02-0.03)
 
 =cut
 
